@@ -249,9 +249,9 @@ gnutls_x509_crt_set_key (gnutls_x509_crt_t crt, gnutls_x509_privkey_t key)
  * @crt: a certificate of type #gnutls_x509_crt_t
  * @crq: holds a certificate request
  *
- * This function will set the name and public parameters from the
- * given certificate request to the certificate. Only RSA keys are
- * currently supported.
+ * This function will set the name and public parameters as well as
+ * the extensions from the given certificate request to the certificate. 
+ * Only RSA keys are currently supported.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
@@ -260,7 +260,6 @@ int
 gnutls_x509_crt_set_crq (gnutls_x509_crt_t crt, gnutls_x509_crq_t crq)
 {
   int result;
-  int pk_algorithm;
 
   if (crt == NULL || crq == NULL)
     {
@@ -268,8 +267,6 @@ gnutls_x509_crt_set_crq (gnutls_x509_crt_t crt, gnutls_x509_crq_t crq)
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  pk_algorithm = gnutls_x509_crq_get_pk_algorithm (crq, NULL);
-
   result = asn1_copy_node (crt->cert, "tbsCertificate.subject",
 			   crq->crq, "certificationRequestInfo.subject");
   if (result != ASN1_SUCCESS)
@@ -290,6 +287,81 @@ gnutls_x509_crt_set_crq (gnutls_x509_crt_t crt, gnutls_x509_crq_t crq)
   return 0;
 }
 
+/**
+ * gnutls_x509_crt_set_crq_extensions - Associate the Certificate with a request
+ * @crt: a certificate of type #gnutls_x509_crt_t
+ * @crq: holds a certificate request
+ *
+ * This function will set extensions from the given request to the
+ * certificate.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 2.8.0
+ **/
+int
+gnutls_x509_crt_set_crq_extensions (gnutls_x509_crt_t crt,
+				    gnutls_x509_crq_t crq)
+{
+  int result, i;
+  char oid[MAX_OID_SIZE];
+  size_t oid_size;
+  opaque extensions[MAX_CRQ_EXTENSIONS_SIZE];
+  size_t extensions_size = sizeof (extensions);
+  unsigned int critical;
+  gnutls_datum ext;
+
+  if (crt == NULL || crq == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  for (i = 0;; i++)
+    {
+      oid_size = sizeof (oid);
+      result =
+	gnutls_x509_crq_get_extension_info (crq, i, oid, &oid_size,
+					    &critical);
+      if (result < 0)
+	{
+	  if (result == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+	    break;
+
+	  gnutls_assert ();
+	  return result;
+	}
+
+      extensions_size = sizeof (extensions);
+      result =
+	gnutls_x509_crq_get_extension_data (crq, i, extensions,
+					    &extensions_size);
+
+      if (result < 0)
+	{
+	  gnutls_assert ();
+	  return result;
+	}
+
+      ext.data = extensions;
+      ext.size = extensions_size;
+
+      result = _gnutls_x509_crt_set_extension (crt, oid, &ext, critical);
+      if (result < 0)
+	{
+	  gnutls_assert ();
+	  return result;
+	}
+
+    }
+
+  if (i > 0)
+    crt->use_extensions = 1;
+
+  return 0;
+}
+
 /**
  * gnutls_x509_crt_set_extension_by_oid - Set an arbitrary extension
  * @crt: a certificate of type #gnutls_x509_crt_t
@@ -478,15 +550,17 @@ gnutls_x509_crt_set_subject_alternative_name (gnutls_x509_crt_t crt,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  /* only handle text extensions */  
-  if (type != GNUTLS_SAN_DNSNAME && type != GNUTLS_SAN_RFC822NAME && 
-    type != GNUTLS_SAN_URI) 
+  /* only handle text extensions */
+  if (type != GNUTLS_SAN_DNSNAME && type != GNUTLS_SAN_RFC822NAME &&
+      type != GNUTLS_SAN_URI)
     {
-      gnutls_assert();
+      gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  return gnutls_x509_crt_set_subject_alt_name( crt, type, data_string, strlen(data_string), GNUTLS_FSAN_SET);
+  return gnutls_x509_crt_set_subject_alt_name (crt, type, data_string,
+					       strlen (data_string),
+					       GNUTLS_FSAN_SET);
 }
 
 /**
@@ -827,7 +901,9 @@ gnutls_x509_crt_set_crl_dist_points (gnutls_x509_crt_t crt,
 				     type, const void *data_string,
 				     unsigned int reason_flags)
 {
-  return gnutls_x509_crt_set_crl_dist_points2( crt, type, data_string, strlen(data_string),reason_flags);
+  return gnutls_x509_crt_set_crl_dist_points2 (crt, type, data_string,
+					       strlen (data_string),
+					       reason_flags);
 }
 
 /**
