@@ -1,15 +1,6 @@
 #include <stdbool.h>
 #include <gnutls/x509.h>
 
-extern char *organization, *unit, *locality, *state;
-extern char *cn, *challenge_password, *password, *pkcs9_email, *country;
-extern char *dns_name, *email, *crl_dist_points, *pkcs12_key_name;
-extern int serial, expiration_days, ca, tls_www_client, tls_www_server,
-  signing_key;
-extern int encryption_key, cert_sign_key, crl_sign_key, code_sign_key,
-  ocsp_sign_key;
-extern int time_stamping_key, crl_next_update;
-
 void cfg_init (void);
 int template_parse (const char *template);
 
@@ -37,7 +28,9 @@ void get_oid_crt_set (gnutls_x509_crt_t crt);
 int get_serial (void);
 int get_days (void);
 int get_ca_status (void);
+int get_crl_number (void);
 int get_path_len (void);
+int get_crq_extensions_status(void);
 const char *get_pkcs12_key_name (void);
 int get_tls_client_status (void);
 int get_tls_server_status (void);
@@ -49,9 +42,9 @@ int get_crl_sign_status (void);
 int get_cert_sign_status (void);
 int get_encrypt_status (int server);
 int get_sign_status (int server);
-void get_ip_addr_set (gnutls_x509_crt_t crt);
-void get_dns_name_set (gnutls_x509_crt_t crt);
-void get_email_set (gnutls_x509_crt_t crt);
+void get_ip_addr_set (int type, void* crt);
+void get_dns_name_set (int type, void* crt);
+void get_email_set (int type, void* crt);
 
 
 void get_cn_crq_set (gnutls_x509_crq_t crq);
