@@ -30,6 +30,7 @@
 #include <gnutls/openpgp.h>
 #include <time.h>
 #include "certtool-gaa.h"
+#include "certtool-common.h"
 #include <gnutls/pkcs12.h>
 #include <unistd.h>
 #include <certtool-cfg.h>
@@ -48,6 +49,7 @@
 static void print_crl_info (gnutls_x509_crl_t crl, FILE * out);
 int generate_prime (int bits, int how);
 void pkcs7_info (void);
+void crq_info (void);
 void smime_to_pkcs7 (void);
 void pkcs12_info (void);
 void generate_pkcs12 (void);
@@ -70,7 +72,7 @@ static void print_certificate_info (gnutls_x509_crt_t crt, FILE * out,
 static void gaa_parser (int argc, char **argv);
 void generate_self_signed (void);
 void generate_request (void);
-gnutls_x509_crt_t *load_cert_list (int mand, int *size);
+gnutls_x509_crt_t *load_cert_list (int mand, size_t * size);
 
 static void print_hex_datum (gnutls_datum_t * dat);
 
@@ -350,6 +352,14 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
     fprintf (stderr, "\n\nExtensions.\n");
 
   /* do not allow extensions on a v1 certificate */
+  if (crq && get_crq_extensions_status () != 0)
+    {
+      result = gnutls_x509_crt_set_crq_extensions (crt, crq);
+      if (result < 0)
+	error (EXIT_FAILURE, 0, "set_crq: %s", gnutls_strerror (result));
+    }
+
+  /* append additional extensions */
   if (info.v1_cert == 0)
     {
 
@@ -406,8 +416,8 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
 
 	  if (!proxy)
 	    {
-	      get_dns_name_set( crt);
-	      get_ip_addr_set( crt);
+	      get_dns_name_set (TYPE_CRT, crt);
+	      get_ip_addr_set (TYPE_CRT, crt);
 	    }
 
 	  result =
@@ -418,7 +428,7 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
 	}
       else if (!proxy)
 	{
-	  get_email_set( crt);
+	  get_email_set (TYPE_CRT, crt);
 	}
 
       if (!ca_status || server)
@@ -550,13 +560,15 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
 }
 
 static gnutls_x509_crl_t
-generate_crl (void)
+generate_crl (gnutls_x509_crt_t ca_crt)
 {
   gnutls_x509_crl_t crl;
   gnutls_x509_crt_t *crts;
-  int size;
-  int days, result, i;
+  size_t size;
+  int days, result;
+  unsigned int i;
   time_t now = time (NULL);
+  unsigned int number;
 
   result = gnutls_x509_crl_init (&crl);
   if (result < 0)
@@ -586,6 +598,39 @@ generate_crl (void)
   if (result < 0)
     error (EXIT_FAILURE, 0, "set_version: %s", gnutls_strerror (result));
 
+  /* Authority Key ID.
+   */
+  if (ca_crt != NULL)
+    {
+      size = sizeof (buffer);
+      result = gnutls_x509_crt_get_subject_key_id (ca_crt, buffer,
+						   &size, NULL);
+      if (result < 0)
+	{
+	  size = sizeof (buffer);
+	  result = gnutls_x509_crt_get_key_id (ca_crt, 0, buffer, &size);
+	}
+      if (result >= 0)
+	{
+	  result = gnutls_x509_crl_set_authority_key_id (crl, buffer, size);
+	  if (result < 0)
+	    error (EXIT_FAILURE, 0, "set_authority_key_id: %s",
+		   gnutls_strerror (result));
+	}
+    }
+
+  number = get_crl_number ();
+
+  buffer[4] = number & 0xff;
+  buffer[3] = (number >> 8) & 0xff;
+  buffer[2] = (number >> 16) & 0xff;
+  buffer[1] = (number >> 24) & 0xff;
+  buffer[0] = 0;
+
+  result = gnutls_x509_crl_set_number (crl, buffer, 5);
+  if (result < 0)
+    error (EXIT_FAILURE, 0, "set_number: %s", gnutls_strerror (result));
+
   return crl;
 }
 
@@ -723,7 +768,7 @@ generate_signed_crl (void)
 
   ca_key = load_ca_private_key ();
   ca_crt = load_ca_cert ();
-  crl = generate_crl ();
+  crl = generate_crl (ca_crt);
 
   fprintf (stderr, "\n");
 
@@ -782,19 +827,19 @@ update_signed_certificate (void)
 static FILE *
 safe_open_rw (const char *file)
 {
-  mode_t oldmask;
+  mode_t omask = 0;
   FILE *fh;
 
   if (info.privkey_op != 0)
     {
-      oldmask = umask (S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
+      omask = umask (S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
     }
 
   fh = fopen (file, "wb");
 
   if (info.privkey_op != 0)
     {
-      umask (oldmask);
+      umask (omask);
     }
 
   return fh;
@@ -869,8 +914,11 @@ gaa_parser (int argc, char **argv)
       template_parse (info.template);
     }
 
+  if (info.quick_random != 0)
+    gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+
 #ifdef gcry_fips_mode_active
-  if (gcry_fips_mode_active())
+  if (gcry_fips_mode_active ())
     {
       ret = gnutls_register_md5_handler ();
       if (ret)
@@ -888,79 +936,79 @@ gaa_parser (int argc, char **argv)
   if ((ret = gnutls_global_init_extra ()) < 0)
     error (EXIT_FAILURE, 0, "global_init_extra: %s", gnutls_strerror (ret));
 
-  if (info.quick_random != 0)
-    gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
-
   switch (info.action)
     {
-    case 0:
+    case ACTION_SELF_SIGNED:
       generate_self_signed ();
       break;
-    case 1:
+    case ACTION_GENERATE_PRIVKEY:
       generate_private_key ();
       break;
-    case 2:
+    case ACTION_CERT_INFO:
       certificate_info ();
       break;
-    case 3:
+    case ACTION_GENERATE_REQUEST:
       generate_request ();
       break;
-    case 4:
+    case ACTION_GENERATE_CERTIFICATE:
       generate_signed_certificate ();
       break;
-    case 5:
+    case ACTION_VERIFY_CHAIN:
       verify_chain ();
       break;
-    case 6:
+    case ACTION_PRIVKEY_INFO:
       privkey_info ();
       break;
-    case 7:
+    case ACTION_UPDATE_CERTIFICATE:
       update_signed_certificate ();
       break;
-    case 8:
+    case ACTION_TO_PKCS12:
       generate_pkcs12 ();
       break;
-    case 9:
+    case ACTION_PKCS12_INFO:
       pkcs12_info ();
       break;
-    case 10:
+    case ACTION_GENERATE_DH:
       generate_prime (info.bits, 1);
       break;
-    case 16:
+    case ACTION_GET_DH:
       generate_prime (info.bits, 0);
       break;
-    case 11:
+    case ACTION_CRL_INFO:
       crl_info ();
       break;
-    case 12:
+    case ACTION_P7_INFO:
       pkcs7_info ();
       break;
-    case 13:
+    case ACTION_GENERATE_CRL:
       generate_signed_crl ();
       break;
-    case 14:
+    case ACTION_VERIFY_CRL:
       verify_crl ();
       break;
-    case 15:
+    case ACTION_SMIME_TO_P7:
       smime_to_pkcs7 ();
       break;
-    case 17:
+    case ACTION_GENERATE_PROXY:
       generate_proxy_certificate ();
       break;
-    case 18:
+    case ACTION_GENERATE_PKCS8:
       generate_pkcs8 ();
       break;
 #ifdef ENABLE_OPENPGP
-    case 19:
+    case ACTION_PGP_INFO:
       pgp_certificate_info ();
       break;
-    case 20:
+    case ACTION_PGP_PRIVKEY_INFO:
       pgp_privkey_info ();
       break;
-    case 21:
+    case ACTION_RING_INFO:
       pgp_ring_info ();
       break;
 #endif
+    case ACTION_REQUEST:
+      crq_info ();
+      break;
     default:
       gaa_help ();
       exit (0);
@@ -1022,8 +1070,8 @@ certificate_info (void)
       if (ret < 0)
 	error (EXIT_FAILURE, 0, "Export error: %s", gnutls_strerror (ret));
       fwrite (buffer, 1, size, outfile);
-      
-      gnutls_x509_crt_deinit( crt[i]);
+
+      gnutls_x509_crt_deinit (crt[i]);
     }
 }
 
@@ -1279,17 +1327,17 @@ print_hex_datum (gnutls_datum_t * dat)
 static void
 print_certificate_info (gnutls_x509_crt_t crt, FILE * out, unsigned int all)
 {
-  gnutls_datum_t info;
+  gnutls_datum_t cinfo;
   int ret;
 
   if (all)
-    ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_FULL, &info);
+    ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_FULL, &cinfo);
   else
-    ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_UNSIGNED_FULL, &info);
+    ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_UNSIGNED_FULL, &cinfo);
   if (ret == 0)
     {
-      fprintf (out, "%s\n", info.data);
-      gnutls_free (info.data);
+      fprintf (out, "%s\n", cinfo.data);
+      gnutls_free (cinfo.data);
     }
 
   if (out == stderr && batch == 0)	/* interactive */
@@ -1302,17 +1350,17 @@ print_certificate_info (gnutls_x509_crt_t crt, FILE * out, unsigned int all)
 static void
 print_crl_info (gnutls_x509_crl_t crl, FILE * out)
 {
-  gnutls_datum_t info;
+  gnutls_datum_t cinfo;
   int ret;
   size_t size;
 
-  ret = gnutls_x509_crl_print (crl, GNUTLS_CRT_PRINT_FULL, &info);
+  ret = gnutls_x509_crl_print (crl, GNUTLS_CRT_PRINT_FULL, &cinfo);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "crl_print: %s", gnutls_strerror (ret));
 
-  fprintf (out, "%s\n", info.data);
+  fprintf (out, "%s\n", cinfo.data);
 
-  gnutls_free (info.data);
+  gnutls_free (cinfo.data);
 
   size = sizeof (buffer);
   ret = gnutls_x509_crl_export (crl, GNUTLS_X509_FMT_PEM, buffer, &size);
@@ -1352,6 +1400,59 @@ crl_info (void)
   gnutls_x509_crl_deinit (crl);
 }
 
+static void
+print_crq_info (gnutls_x509_crq_t crq, FILE * out)
+{
+  gnutls_datum_t cinfo;
+  int ret;
+  size_t size;
+
+  ret = gnutls_x509_crq_print (crq, GNUTLS_CRT_PRINT_FULL, &cinfo);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "crq_print: %s", gnutls_strerror (ret));
+
+  fprintf (out, "%s\n", cinfo.data);
+
+  gnutls_free (cinfo.data);
+
+  size = sizeof (buffer);
+  ret = gnutls_x509_crq_export (crq, GNUTLS_X509_FMT_PEM, buffer, &size);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "crq_export: %s", gnutls_strerror (ret));
+
+  fwrite (buffer, 1, size, outfile);
+}
+
+void
+crq_info (void)
+{
+  gnutls_x509_crq_t crq;
+  int ret;
+  size_t size;
+  gnutls_datum_t pem;
+
+  ret = gnutls_x509_crq_init (&crq);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "crq_init: %s", gnutls_strerror (ret));
+
+  pem.data = fread_file (infile, &size);
+  pem.size = size;
+
+  if (!pem.data)
+    error (EXIT_FAILURE, errno, "%s", info.infile ? info.infile :
+	   "standard input");
+
+  ret = gnutls_x509_crq_import (crq, &pem, info.incert_format);
+
+  free (pem.data);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "Import error: %s", gnutls_strerror (ret));
+
+  print_crq_info (crq, outfile);
+
+  gnutls_x509_crq_deinit (crq);
+}
+
 void
 privkey_info (void)
 {
@@ -1377,19 +1478,13 @@ privkey_info (void)
   /* If we failed to import the certificate previously try PKCS #8 */
   if (info.pkcs8 || ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR)
     {
-      /* first try to import the key without asking any password */
+      if (info.pass)
+	pass = info.pass;
+      else
+	pass = get_pass ();
       ret =
 	gnutls_x509_privkey_import_pkcs8 (key, &pem, info.incert_format,
-					  NULL, GNUTLS_PKCS_PLAIN);
-      if (ret < 0)
-	{
-	  if (info.pass)
-	    pass = info.pass;
-	  else
-	    pass = get_pass ();
-	  ret = gnutls_x509_privkey_import_pkcs8 (key, &pem,
-						  info.incert_format, pass, 0);
-	}
+					  pass, 0);
     }
   if (ret < 0)
     error (EXIT_FAILURE, 0, "Import error: %s", gnutls_strerror (ret));
@@ -1416,15 +1511,15 @@ privkey_info (void)
 	fprintf (stderr, "Error in key RSA data export: %s\n",
 		 gnutls_strerror (ret));
       else
-        {
+	{
 	  print_rsa_pkey (&m, &e, &d, &p, &q, &u);
-          gnutls_free(m.data);
-          gnutls_free(e.data);
-          gnutls_free(d.data);
-          gnutls_free(p.data);
-          gnutls_free(q.data);
-          gnutls_free(u.data);
-        }
+	  gnutls_free (m.data);
+	  gnutls_free (e.data);
+	  gnutls_free (d.data);
+	  gnutls_free (p.data);
+	  gnutls_free (q.data);
+	  gnutls_free (u.data);
+	}
     }
   else if (ret == GNUTLS_PK_DSA)
     {
@@ -1435,14 +1530,14 @@ privkey_info (void)
 	fprintf (stderr, "Error in key DSA data export: %s\n",
 		 gnutls_strerror (ret));
       else
-        {
+	{
 	  print_dsa_pkey (&x, &y, &p, &q, &g);
-          gnutls_free(x.data);
-          gnutls_free(y.data);
-          gnutls_free(p.data);
-          gnutls_free(q.data);
-          gnutls_free(g.data);
-        }
+	  gnutls_free (x.data);
+	  gnutls_free (y.data);
+	  gnutls_free (p.data);
+	  gnutls_free (q.data);
+	  gnutls_free (g.data);
+	}
     }
 
   fprintf (outfile, "\n");
@@ -1647,7 +1742,7 @@ gnutls_x509_crt_t
 load_cert (int mand)
 {
   gnutls_x509_crt_t *crt;
-  int size;
+  size_t size;
 
   crt = load_cert_list (mand, &size);
 
@@ -1659,7 +1754,7 @@ load_cert (int mand)
 /* Loads a certificate list
  */
 gnutls_x509_crt_t *
-load_cert_list (int mand, int *crt_size)
+load_cert_list (int mand, size_t * crt_size)
 {
   FILE *fd;
   static gnutls_x509_crt_t crt[MAX_CERTS];
@@ -1721,7 +1816,7 @@ load_cert_list (int mand, int *crt_size)
 
       (*crt_size)++;
     }
-  fprintf (stderr, "Loaded %d certificates.\n", *crt_size);
+  fprintf (stderr, "Loaded %d certificates.\n", (int) *crt_size);
 
   return crt;
 }
@@ -1734,9 +1829,9 @@ generate_request (void)
 {
   gnutls_x509_crq_t crq;
   gnutls_x509_privkey_t key;
-  int ret;
+  int ret, ca_status, path_len;
   const char *pass;
-  size_t size;
+  unsigned int usage = 0;
 
   fprintf (stderr, "Generating a PKCS #10 certificate request...\n");
 
@@ -1765,15 +1860,105 @@ generate_request (void)
   get_uid_crq_set (crq);
   get_oid_crq_set (crq);
 
+  get_dns_name_set (TYPE_CRQ, crq);
+  get_ip_addr_set (TYPE_CRQ, crq);
+  get_email_set (TYPE_CRQ, crq);
+
   pass = get_challenge_pass ();
 
-  if (pass != NULL)
+  if (pass != NULL && pass[0] != 0)
     {
       ret = gnutls_x509_crq_set_challenge_password (crq, pass);
       if (ret < 0)
 	error (EXIT_FAILURE, 0, "set_pass: %s", gnutls_strerror (ret));
     }
 
+  ca_status = get_ca_status ();
+  if (ca_status)
+    path_len = get_path_len ();
+  else
+    path_len = -1;
+
+  ret = gnutls_x509_crq_set_basic_constraints (crq, ca_status, path_len);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "set_basic_constraints: %s",
+	   gnutls_strerror (ret));
+
+
+  ret = get_sign_status (1);
+  if (ret)
+    usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
+
+  ret = get_encrypt_status (1);
+  if (ret)
+    usage |= GNUTLS_KEY_KEY_ENCIPHERMENT;
+  else
+    usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
+
+  if (ca_status)
+    {
+      ret = get_cert_sign_status ();
+      if (ret)
+	usage |= GNUTLS_KEY_KEY_CERT_SIGN;
+
+      ret = get_crl_sign_status ();
+      if (ret)
+	usage |= GNUTLS_KEY_CRL_SIGN;
+
+      ret = get_code_sign_status ();
+      if (ret)
+	{
+	  ret =
+	    gnutls_x509_crq_set_key_purpose_oid (crq,
+						 GNUTLS_KP_CODE_SIGNING, 0);
+	  if (ret < 0)
+	    error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
+	}
+
+      ret = get_ocsp_sign_status ();
+      if (ret)
+	{
+	  ret =
+	    gnutls_x509_crq_set_key_purpose_oid (crq,
+						 GNUTLS_KP_OCSP_SIGNING, 0);
+	  if (ret < 0)
+	    error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
+	}
+
+      ret = get_time_stamp_status ();
+      if (ret)
+	{
+	  ret =
+	    gnutls_x509_crq_set_key_purpose_oid (crq,
+						 GNUTLS_KP_TIME_STAMPING, 0);
+	  if (ret < 0)
+	    error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
+	}
+
+    }
+
+  ret = gnutls_x509_crq_set_key_usage (crq, usage);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "key_usage: %s", gnutls_strerror (ret));
+
+  ret = get_tls_client_status ();
+  if (ret != 0)
+    {
+      ret = gnutls_x509_crq_set_key_purpose_oid (crq,
+						 GNUTLS_KP_TLS_WWW_CLIENT, 0);
+      if (ret < 0)
+	error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
+    }
+
+  ret = get_tls_server_status ();
+  if (ret != 0)
+    {
+      ret = gnutls_x509_crq_set_key_purpose_oid (crq,
+						 GNUTLS_KP_TLS_WWW_SERVER, 0);
+      if (ret < 0)
+	error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
+    }
+
   ret = gnutls_x509_crq_set_key (crq, key);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "set_key: %s", gnutls_strerror (ret));
@@ -1782,12 +1967,7 @@ generate_request (void)
   if (ret < 0)
     error (EXIT_FAILURE, 0, "sign: %s", gnutls_strerror (ret));
 
-  size = sizeof (buffer);
-  ret = gnutls_x509_crq_export (crq, info.outcert_format, buffer, &size);
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "export: %s", gnutls_strerror (ret));
-
-  fwrite (buffer, 1, size, outfile);
+  print_crq_info (crq, outfile);
 
   gnutls_x509_crq_deinit (crq);
   gnutls_x509_privkey_deinit (key);
@@ -2088,16 +2268,16 @@ print_verification_res (gnutls_x509_crt_t crt,
 void
 verify_chain (void)
 {
-  char *buffer;
+  char *buf;
   size_t size;
 
-  buffer = fread_file (infile, &size);
-  if (buffer == NULL)
+  buf = fread_file (infile, &size);
+  if (buf == NULL)
     error (EXIT_FAILURE, errno, "reading chain");
 
-  buffer[size] = 0;
+  buf[size] = 0;
 
-  _verify_x509_mem (buffer, size);
+  _verify_x509_mem (buf, size);
 
 }
 
@@ -2246,14 +2426,13 @@ generate_pkcs12 (void)
   int result;
   size_t size;
   gnutls_datum_t data;
-  const char *password;
+  const char *pass;
   const char *name;
-  unsigned int flags;
+  unsigned int flags, i;
   gnutls_datum_t key_id;
   unsigned char _key_id[20];
-  int index;
-  int ncrts;
-  int i;
+  int indx;
+  size_t ncrts;
 
   fprintf (stderr, "Generating a PKCS #12 structure...\n");
 
@@ -2267,9 +2446,9 @@ generate_pkcs12 (void)
     error (EXIT_FAILURE, 0, "pkcs12_init: %s", gnutls_strerror (result));
 
   if (info.pass)
-    password = info.pass;
+    pass = info.pass;
   else
-    password = get_pass ();
+    pass = get_pass ();
 
   for (i = 0; i < ncrts; i++)
     {
@@ -2284,9 +2463,9 @@ generate_pkcs12 (void)
 	error (EXIT_FAILURE, 0, "set_crt[%d]: %s", i,
 	       gnutls_strerror (result));
 
-      index = result;
+      indx = result;
 
-      result = gnutls_pkcs12_bag_set_friendly_name (bag, index, name);
+      result = gnutls_pkcs12_bag_set_friendly_name (bag, indx, name);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_set_friendly_name: %s",
 	       gnutls_strerror (result));
@@ -2300,7 +2479,7 @@ generate_pkcs12 (void)
       key_id.data = _key_id;
       key_id.size = size;
 
-      result = gnutls_pkcs12_bag_set_key_id (bag, index, &key_id);
+      result = gnutls_pkcs12_bag_set_key_id (bag, indx, &key_id);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_set_key_id: %s",
 	       gnutls_strerror (result));
@@ -2310,7 +2489,7 @@ generate_pkcs12 (void)
       else
 	flags = GNUTLS_PKCS8_USE_PKCS12_3DES;
 
-      result = gnutls_pkcs12_bag_encrypt (bag, password, flags);
+      result = gnutls_pkcs12_bag_encrypt (bag, pass, flags);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_encrypt: %s", gnutls_strerror (result));
 
@@ -2335,7 +2514,7 @@ generate_pkcs12 (void)
       size = sizeof (buffer);
       result =
 	gnutls_x509_privkey_export_pkcs8 (key, GNUTLS_X509_FMT_DER,
-					  password, flags, buffer, &size);
+					  pass, flags, buffer, &size);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "key_export: %s", gnutls_strerror (result));
 
@@ -2347,9 +2526,9 @@ generate_pkcs12 (void)
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_set_data: %s", gnutls_strerror (result));
 
-      index = result;
+      indx = result;
 
-      result = gnutls_pkcs12_bag_set_friendly_name (kbag, index, name);
+      result = gnutls_pkcs12_bag_set_friendly_name (kbag, indx, name);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_set_friendly_name: %s",
 	       gnutls_strerror (result));
@@ -2362,7 +2541,7 @@ generate_pkcs12 (void)
       key_id.data = _key_id;
       key_id.size = size;
 
-      result = gnutls_pkcs12_bag_set_key_id (kbag, index, &key_id);
+      result = gnutls_pkcs12_bag_set_key_id (kbag, indx, &key_id);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_set_key_id: %s",
 	       gnutls_strerror (result));
@@ -2372,7 +2551,7 @@ generate_pkcs12 (void)
 	error (EXIT_FAILURE, 0, "set_bag: %s", gnutls_strerror (result));
     }
 
-  result = gnutls_pkcs12_generate_mac (pkcs12, password);
+  result = gnutls_pkcs12_generate_mac (pkcs12, pass);
   if (result < 0)
     error (EXIT_FAILURE, 0, "generate_mac: %s", gnutls_strerror (result));
 
@@ -2488,8 +2667,8 @@ pkcs12_info (void)
   int result;
   size_t size;
   gnutls_datum_t data;
-  const char *password;
-  int index;
+  const char *pass;
+  int indx;
 
   result = gnutls_pkcs12_init (&pkcs12);
   if (result < 0)
@@ -2504,23 +2683,23 @@ pkcs12_info (void)
     error (EXIT_FAILURE, 0, "p12_import: %s", gnutls_strerror (result));
 
   if (info.pass)
-    password = info.pass;
+    pass = info.pass;
   else
-    password = get_pass ();
+    pass = get_pass ();
 
-  result = gnutls_pkcs12_verify_mac (pkcs12, password);
+  result = gnutls_pkcs12_verify_mac (pkcs12, pass);
   if (result < 0)
     error (EXIT_FAILURE, 0, "verify_mac: %s", gnutls_strerror (result));
 
-  index = 0;
+  indx = 0;
 
-  for (index = 0;; index++)
+  for (indx = 0;; indx++)
     {
       result = gnutls_pkcs12_bag_init (&bag);
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_init: %s", gnutls_strerror (result));
 
-      result = gnutls_pkcs12_get_bag (pkcs12, index, bag);
+      result = gnutls_pkcs12_get_bag (pkcs12, indx, bag);
       if (result < 0)
 	break;
 
@@ -2528,7 +2707,7 @@ pkcs12_info (void)
       if (result < 0)
 	error (EXIT_FAILURE, 0, "bag_count: %s", gnutls_strerror (result));
 
-      fprintf (outfile, "BAG #%d\n", index);
+      fprintf (outfile, "BAG #%d\n", indx);
 
       result = gnutls_pkcs12_bag_get_type (bag, 0);
       if (result < 0)
@@ -2539,7 +2718,7 @@ pkcs12_info (void)
 	  fprintf (stderr, "\tType: %s\n", BAGTYPE (result));
 	  fprintf (stderr, "\n\tDecrypting...\n");
 
-	  result = gnutls_pkcs12_bag_decrypt (bag, password);
+	  result = gnutls_pkcs12_bag_decrypt (bag, pass);
 
 	  if (result < 0)
 	    error (EXIT_FAILURE, 0, "bag_decrypt: %s",
@@ -2564,7 +2743,7 @@ pkcs7_info (void)
   int result;
   size_t size;
   gnutls_datum_t data, b64;
-  int index, count;
+  int indx, count;
 
   result = gnutls_pkcs7_init (&pkcs7);
   if (result < 0)
@@ -2589,12 +2768,12 @@ pkcs7_info (void)
   if (count > 0)
     fprintf (outfile, "Number of certificates: %u\n", count);
 
-  for (index = 0; index < count; index++)
+  for (indx = 0; indx < count; indx++)
     {
       fputs ("\n", outfile);
 
       size = sizeof (buffer);
-      result = gnutls_pkcs7_get_crt_raw (pkcs7, index, buffer, &size);
+      result = gnutls_pkcs7_get_crt_raw (pkcs7, indx, buffer, &size);
       if (result < 0)
 	break;
 
@@ -2620,12 +2799,12 @@ pkcs7_info (void)
   if (count > 0)
     fprintf (outfile, "\nNumber of CRLs: %u\n", count);
 
-  for (index = 0; index < count; index++)
+  for (indx = 0; indx < count; indx++)
     {
       fputs ("\n", outfile);
 
       size = sizeof (buffer);
-      result = gnutls_pkcs7_get_crl_raw (pkcs7, index, buffer, &size);
+      result = gnutls_pkcs7_get_crl_raw (pkcs7, indx, buffer, &size);
       if (result < 0)
 	break;
 
