@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008 Free Software Foundation
  * Copyright (C) 2000,2001,2002,2003 Nikos Mavrogiannopoulos
  *
  * This file is part of GNUTLS.
@@ -45,8 +45,6 @@
 #include "read-file.h"
 #include "getpass.h"
 
-#include "getaddrinfo.h"
-
 #include "common.h"
 #include "cli-gaa.h"
 
@@ -547,7 +545,7 @@ handle_error (socket_st * hd, int err)
        */
     }
 
-  check_rehandshake (hd, err);
+  check_rehandshake (hd, ret);
 
   return ret;
 }
@@ -579,7 +577,7 @@ main (int argc, char **argv)
   fd_set rset;
   int maxfd;
   struct timeval tv;
-  int user_term = 0, retval = 0;
+  int user_term = 0;
   socket_st hd;
 
   set_program_name (argv[0]);
@@ -587,7 +585,7 @@ main (int argc, char **argv)
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 
 #ifdef gcry_fips_mode_active
-  if (gcry_fips_mode_active())
+  if (gcry_fips_mode_active ())
     {
       ret = gnutls_register_md5_handler ();
       if (ret)
@@ -661,6 +659,8 @@ main (int argc, char **argv)
 	    printf ("*** This is a resumed session\n");
 	}
 
+
+
       if (resume != 0 && i == 0)
 	{
 
@@ -721,7 +721,6 @@ after_handshake:
 	    {
 	      fprintf (stderr, "*** Handshake has failed\n");
 	      user_term = 1;
-	      retval = 1;
 	      break;
 	    }
 	}
@@ -752,7 +751,6 @@ after_handshake:
 	    {
 	      fprintf (stderr,
 		       "*** Server has terminated the connection abnormally.\n");
-	      retval = 1;
 	      break;
 	    }
 	  else if (ret > 0)
@@ -786,7 +784,6 @@ after_handshake:
 		    {
 		      fprintf (stderr, "*** Handshake has failed\n");
 		      user_term = 1;
-		      retval = 1;
 		      break;
 		    }
 		}
@@ -843,7 +840,7 @@ after_handshake:
 
   gnutls_global_deinit ();
 
-  return retval;
+  return 0;
 }
 
 void
